@@ -3,35 +3,27 @@
   switch(vcov_type,
          "const" = "maximum likelihood",
          "asympt" = {
-           if (!cluster)
-             "HC0 robust (adjusted for estimation of weights)"
-           else
-             "HC0 cluster-robust (adjusted for estimation of weights)"
+           if (cluster) "HC0 cluster-robust (adjusted for estimation of weights)"
+           else"HC0 robust (adjusted for estimation of weights)"
          },
          "HC0" = {
-           if (!cluster)
-             "HC0 robust"
-           else
-             "HC0 cluster-robust"
+           if (cluster) "HC0 cluster-robust"
+           else "HC0 robust"
          },
          "BS" = {
-           if (!cluster)
-             "traditional bootstrap"
-           else
-             "traditional cluster bootstrap"
+           if (cluster) "traditional cluster bootstrap"
+           else "traditional bootstrap"
          },
          "FWB" = {
-           if (!cluster)
-             "fractional weighted bootstrap"
-           else
-             "fractional weighted cluster bootstrap"
+           if (cluster) "fractional weighted cluster bootstrap"
+           else "fractional weighted bootstrap"
          },
          "none" = "none",
          "user-supplied")
 }
 
 # Processes the vcov argument from glm_weightit(); returns vcov_type with attributes
-.process_vcov <- function(vcov = NULL, weightit = NULL, R, fwb.args = list(),
+.process_vcov <- function(vcov = NULL, weightit = NULL, R = 500L, fwb.args = list(),
                           m_est_supported = TRUE) {
   if (is_null(weightit)) {
     if (is_null(vcov)) {
@@ -73,7 +65,7 @@
 
   if (bootstrap) {
     chk::chk_count(R)
-    chk::chk_gt(R, 0)
+    chk::chk_gt(R, 0L)
     attr(vcov, "R") <- R
 
     if (vcov == "FWB") {
@@ -181,7 +173,7 @@
   }
 
   if (is_null(V)) {
-    .err("no variance matrix was found. See the `vcov` argument at `help(\"anova.glm_weightit\")` for details")
+    .err('no variance matrix was found. See the `vcov` argument at `help("anova.glm_weightit")` for details')
   }
 
   if (is_not_null(b1)) {
@@ -214,14 +206,12 @@
 
 # Dispatches computation of vcov; for using in vcov.glm_weightit() and summary.glm_weightit()
 .vcov_glm_weightit.internal <- function(object, vcov. = NULL, ...) {
-  dots <- list(...)
 
   if (is_null(vcov.)) {
-    vcov. <- dots[["type"]]
-    dots[["type"]] <- NULL
+    vcov. <- ...get("type")
   }
 
-  if (is_null(vcov.) && is_null(dots)) {
+  if (is_null(vcov.) && ...length() == 0L) {
     if (is_not_null(object[["vcov"]])) {
       return(.modify_vcov_info(object[["vcov"]],
                                vcov_type = object[["vcov_type"]],
@@ -237,8 +227,8 @@
     return(NULL)
   }
 
-  R <- if_null_then(dots[["R"]], 500)
-  fwb.args <- if_null_then(dots[["fwb.args"]], list())
+  R <- ...get("R", 500L)
+  fwb.args <- ...get("fwb.args", list())
 
   vcov. <- .process_vcov(vcov., object[["weightit"]], R, fwb.args)
 
@@ -247,13 +237,13 @@
   }
 
   cluster <- {
-    if ("cluster" %in% names(dots)) dots[["cluster"]]
+    if ("cluster" %in% ...names()) ...get("cluster")
     else attr(object, "cluster")
   }
 
-  glm_call <- .build_model_call(object, vcov = vcov.)
+  internal_model_call <- .build_internal_model_call(object, vcov = vcov.)
 
-  .compute_vcov(object, object[["weightit"]], vcov., cluster, glm_call)
+  .compute_vcov(object, object[["weightit"]], vcov., cluster, object[["call"]], internal_model_call)
 }
 
 .modify_vcov_info <- function(vcov, vcov_type = NULL, cluster = NULL) {
@@ -280,7 +270,9 @@
                                        quote = FALSE,
                                        right = TRUE,
                                        ...) {
-  if (is_null(d <- dim(x)) || length(d) != 2L) {
+  d <- dim(x)
+
+  if (is_null(d) || length(d) != 2L) {
     .err("'x' must be coefficient matrix/data frame")
   }
 
@@ -305,16 +297,14 @@
       chk::chk_subset(p.ind, seq_col(x))
     }
   }
-  else {
-    if (is_not_null(p.ind)) {
-      .err("`has.Pvalue` set to `FALSE` but `p.ind` is not `NULL`")
-    }
+  else if (is_not_null(p.ind)) {
+    .err("`has.Pvalue` set to `FALSE` but `p.ind` is not `NULL`")
   }
 
   if (is_null(P.values)) {
     scp <- getOption("show.coef.Pvalues")
     if (!is.logical(scp) || is.na(scp)) {
-      .wrn("option \"show.coef.Pvalues\" is invalid: assuming `TRUE`")
+      .wrn('option "show.coef.Pvalues" is invalid: assuming `TRUE`')
       scp <- TRUE
     }
     P.values <- has.Pvalue && scp
@@ -359,18 +349,24 @@
 
   Cf <- array("", dim = d, dimnames = dimnames(xm))
 
-  ok <- !(ina <- is.na(xm))
+  ina <- is.na(xm)
+  ok <- !ina
 
   for (i in zap.ind) {
     xm[, i] <- zapsmall(xm[, i], digits)
   }
 
   if (is_not_null(cs.ind)) {
-    acs <- abs(coef.se <- xm[, cs.ind, drop = FALSE])
-    if (any(ia <- is.finite(acs))) {
-      digmin <- 1 + if (length(acs <- acs[ia & acs != 0]))
-        floor(log10(range(acs[acs != 0], finite = TRUE)))
-      else 0
+    coef.se <- xm[, cs.ind, drop = FALSE]
+    acs <- abs(coef.se)
+    ia <- is.finite(acs)
+    if (any(ia)) {
+      acs <- acs[ia & acs != 0]
+
+      digmin <- {
+        if (is_null(acs)) 1
+        else 1 + floor(log10(range(acs[acs != 0], finite = TRUE)))
+      }
 
       Cf[, cs.ind] <- format(round(coef.se, max(1L, digits - digmin)), digits = digits)
     }
@@ -381,7 +377,8 @@
                             digits = digits)
   }
 
-  if (is_not_null(r.ind <- setdiff(seq_col(xm), c(cs.ind, tst.ind, p.ind)))) {
+  r.ind <- setdiff(seq_col(xm), c(cs.ind, tst.ind, p.ind))
+  if (is_not_null(r.ind)) {
     for (i in r.ind) {
       Cf[, i] <- format(xm[, i], digits = digits)
     }
@@ -399,7 +396,8 @@
 
   x0 <- (xm[okP] == 0) != (as.numeric(x1) == 0)
 
-  if (is_not_null(not.both.0 <- which(x0 & !is.na(x0)))) {
+  not.both.0 <- which(x0 & !is.na(x0))
+  if (is_not_null(not.both.0)) {
     Cf[okP][not.both.0] <- format(xm[okP][not.both.0], digits = max(1L, digits - 1L))
   }
 
@@ -407,14 +405,17 @@
     Cf[ina] <- na.print
   }
 
-  if (any(inan <- is.nan(xm))) {
+  inan <- is.nan(xm)
+  if (any(inan)) {
     Cf[inan] <- "NaN"
   }
 
   if (P.values) {
     chk::chk_flag(signif.stars)
 
-    if (any(okP <- ok[, p.ind])) {
+    okP <- ok[, p.ind]
+
+    if (any(okP)) {
       pv <- as.vector(xm[, p.ind])
       Cf[okP, p.ind] <- format.pval(pv[okP], digits = dig.tst,
                                     eps = eps.Pvalue)
@@ -443,11 +444,14 @@
     chk::chk_flag(signif.legend)
 
     if (signif.legend) {
-      if ((w <- getOption("width")) < nchar(sleg <- attr(Signif, "legend"))) {
-        sleg <- strwrap(sleg, width = w - 2, prefix = space(2))
+      w <- getOption("width")
+      sleg <- attr(Signif, "legend")
+
+      if (w < nchar(sleg)) {
+        sleg <- strwrap(sleg, width = w - 2L, prefix = space(2L))
       }
 
-      cat0("---\nSignif. codes:  ", sleg, fill = w + 4 + max(nchar(sleg, "bytes") - nchar(sleg)))
+      cat0("---\nSignif. codes:  ", sleg, fill = w + 4L + max(nchar(sleg, "bytes") - nchar(sleg)))
     }
   }
 
@@ -455,8 +459,8 @@
 }
 
 # Constructs a model call, used in .compute_vcov() to compute variance
-.build_model_call <- function(object = NULL, model = "glm", model_call,
-                              weightit = NULL, vcov = NULL, br = FALSE) {
+.build_internal_model_call <- function(object = NULL, model = "glm", model_call,
+                                       weightit = NULL, vcov = NULL, br = FALSE) {
 
   if (is_not_null(object)) {
     model <- {
@@ -464,6 +468,7 @@
       else if (inherits(object, "multinom_weightit")) "multinom"
       else if (inherits(object, "ordinal_weightit")) "ordinal"
       else if (inherits(object, "glm_weightit")) "glm"
+      else if (inherits(object, "lm_weightit")) "lm"
       else .err("can't build model call. This is probably a bug")
     }
 
@@ -479,7 +484,7 @@
       vcov <- object[["vcov_type"]]
     }
 
-    br <- object[["br"]]
+    br <- isTRUE(object[["br"]])
   }
 
   if (is_not_null(weightit)) {
@@ -488,14 +493,14 @@
     }
 
     if (is_null(weightit[["s.weights"]])) {
-      weightit[["s.weights"]] <- rep(1, nobs(weightit))
+      weightit[["s.weights"]] <- rep.int(1, nobs(weightit))
     }
   }
 
   if (model == "glm") {
     chk::chk_flag(br)
 
-    model_call[[1]] <- quote(stats::glm)
+    model_call[[1L]] <- quote(stats::glm)
 
     if (is_not_null(weightit)) {
       model_call$weights <- weightit[["weights"]] * weightit[["s.weights"]]
@@ -503,6 +508,7 @@
     model_call$x <- TRUE
     model_call$y <- TRUE
     model_call$model <- TRUE
+    model_call$na.action <- "na.fail"
 
     if (br) {
       rlang::check_installed("brglm2")
@@ -516,8 +522,22 @@
 
     model_call[setdiff(names(model_call), c(names(formals(stats::glm)), names(formals(ctrl))))] <- NULL
   }
+  else if (model == "lm") {
+    model_call[[1L]] <- quote(stats::glm)
+
+    if (is_not_null(weightit)) {
+      model_call$weights <- weightit[["weights"]] * weightit[["s.weights"]]
+    }
+    model_call$x <- TRUE
+    model_call$y <- TRUE
+    model_call$model <- TRUE
+    model_call$na.action <- "na.fail"
+    model_call$family <- "gaussian"
+
+    model_call[setdiff(names(model_call), c(names(formals(stats::glm)), names(formals(stats::glm.control))))] <- NULL
+  }
   else if (model == "ordinal") {
-    model_call[[1]] <- .ordinal_weightit
+    model_call[[1L]] <- .ordinal_weightit
     model_call[setdiff(names(model_call), names(formals(.ordinal_weightit)))] <- NULL
 
     if (is_not_null(weightit)) {
@@ -529,7 +549,7 @@
     model_call$hess <- vcov %nin% c("none", "BS", "FWB")
   }
   else if (model == "multinom") {
-    model_call[[1]] <- .multinom_weightit
+    model_call[[1L]] <- .multinom_weightit
     model_call[setdiff(names(model_call), names(formals(.multinom_weightit)))] <- NULL
 
     if (is_not_null(weightit)) {
@@ -541,7 +561,7 @@
     model_call$hess <- vcov %nin% c("none", "BS", "FWB")
   }
   else if (model == "coxph") {
-    model_call[[1]] <- quote(survival::coxph)
+    model_call[[1L]] <- quote(survival::coxph)
 
     if (is_not_null(weightit)) {
       model_call$weights <- weightit[["weights"]] * weightit[["s.weights"]]
@@ -559,7 +579,13 @@
 }
 
 # Computes variance from model fit and call; used in glm_weightit()
-.compute_vcov <- function(fit, weightit = NULL, vcov, cluster = NULL, glm_call) {
+.compute_vcov <- function(fit, weightit = NULL, vcov, cluster = NULL, model_call,
+                          internal_model_call) {
+  # Check missing
+  if (is_not_null(fit[["na.action"]])) {
+    .err("missing values are not allowed in the model variables")
+  }
+
   if (is_not_null(cluster) && vcov %in% c("none", "const")) {
     .wrn("`cluster` is not used when `vcov = %s`", add_quotes(vcov))
   }
@@ -577,21 +603,21 @@
         fit[["hessian"]] <- .get_hess_ordinal(fit)
       }
 
-      V <- solve(-fit[["hessian"]])
+      V <- .solve_hessian(-fit[["hessian"]])
     }
     else if (inherits(fit, "multinom_weightit")) {
       if (is_null(fit[["hessian"]])) {
         fit[["hessian"]] <- .get_hess_multinom(fit)
       }
 
-      V <- solve(-fit[["hessian"]])
+      V <- .solve_hessian(-fit[["hessian"]])
     }
     else if (inherits(fit, "coxph_weightit")) {
       V <- if_null_then(fit[["naive.var"]], fit[["var"]])
     }
     else {
       .declass <- function(obj) {
-        class(obj) <- setdiff(class(obj), "glm_weightit")
+        class(obj) <- setdiff(class(obj), c("glm_weightit", "lm_weightit"))
         obj
       }
 
@@ -614,6 +640,10 @@
     W <- SW <- NULL
   }
 
+  if (is_null(W)) {
+    W <- rep.int(1, length(Y))
+  }
+
   if (is_null(SW)) {
     SW <- rep.int(1, length(Y))
   }
@@ -621,7 +651,7 @@
   offset <- if_null_then(fit[["offset"]], rep.int(0, length(Y)))
 
   if (any(aliased)) {
-    if (!is_null(attr(fit[["qr"]][["qr"]], "aliased"))) {
+    if (is_not_null(attr(fit[["qr"]][["qr"]], "aliased"))) {
       Xout <- Xout[, !attr(fit[["qr"]][["qr"]], "aliased"), drop = FALSE]
     }
     else {
@@ -630,8 +660,6 @@
     bout <- bout[!aliased]
   }
 
-  pout <- sum(!aliased)
-
   if (is_not_null(cluster) && vcov %in% c("asympt", "HC0", "BS", "FWB")) {
     if (inherits(cluster, "formula")) {
       cluster_tmp <- expand.model.frame(fit, cluster, na.expand = FALSE)
@@ -651,7 +679,7 @@
     if (p > 1L) {
       clu <- lapply(seq_len(p), function(i) utils::combn(seq_len(p), i, simplify = FALSE))
       clu <- unlist(clu, recursive = FALSE)
-      sign <- vapply(clu, function(i) (-1)^(length(i) + 1), numeric(1L))
+      sgn <- vapply(clu, function(i) (-1)^(length(i) + 1L), numeric(1L))
       paste_ <- function(...) paste(..., sep = "_")
       for (i in (p + 1L):length(clu)) {
         cluster <- cbind(cluster, Reduce(paste_, unclass(cluster[, clu[[i]]])))
@@ -659,13 +687,13 @@
     }
     else {
       clu <- list(1)
-      sign <- 1
+      sgn <- 1
     }
 
     #Small sample adjustment (setting cadjust = TRUE in vcovCL)
     g <- vapply(seq_along(clu), function(i) {
       if (is.factor(cluster[[i]])) {
-        length(levels(cluster[[i]]))
+        nlevels(cluster[[i]])
       }
       else {
         length(unique(cluster[[i]]))
@@ -677,9 +705,9 @@
     R <- attr(vcov, "R")
     fwb.args <- attr(vcov, "fwb.args")
 
-    glm_call$x <- FALSE
-    glm_call$y <- FALSE
-    glm_call$model <- FALSE
+    internal_model_call$x <- FALSE
+    internal_model_call$y <- FALSE
+    internal_model_call$model <- FALSE
 
     if (is_not_null(weightit)) {
       wcall <- weightit[["call"]]
@@ -695,30 +723,17 @@
       if (is_not_null(weightit)) {
         wcall$s.weights <- SW * w
 
-        withCallingHandlers({
-          weightit_boot <- eval(wcall, wenv)
-        },
-        warning = function(w) {
-          w <- conditionMessage(w)
-          if (!startsWith(tolower(w), "some extreme weights were generated"))
-            .wrn("(from `weightit()`) ", w, tidy = FALSE)
-          invokeRestart("muffleWarning")
-        })
+        weightit_boot <- .eval_fit(wcall, envir = wenv,
+                                   warnings = c("some extreme weights were generated" = NA))
 
-        glm_call$weights <- weightit_boot[["weights"]] * SW * w
+        internal_model_call$weights <- weightit_boot[["weights"]] * SW * w
       }
       else {
-        glm_call$weights <- SW * w
+        internal_model_call$weights <- SW * w
       }
 
-      withCallingHandlers({
-        fit_boot <- eval(glm_call, genv)
-      },
-      warning = function(w) {
-        w <- conditionMessage(w)
-        if (w != "non-integer #successes in a binomial glm!") .wrn("(from `glm()`) ", w, tidy = FALSE)
-        invokeRestart("muffleWarning")
-      })
+      fit_boot <- .eval_fit(internal_model_call, envir = genv,
+                            warnings = c("non-integer" = NA))
 
       fit_boot[["coefficients"]]
     }
@@ -727,7 +742,9 @@
     fwb.args$data <- data.frame(SW)
     fwb.args$statistic <- fwbfun
     fwb.args$R <- R
-    if (is_null(fwb.args$verbose)) fwb.args$verbose <- FALSE
+    if (is_null(fwb.args$verbose)) {
+      fwb.args$verbose <- FALSE
+    }
     fwb.args <- fwb.args[names(fwb.args) %in% names(formals(fwb::fwb))]
 
     if (is_null(cluster)) {
@@ -739,16 +756,16 @@
       for (i in seq_along(clu)) {
         fwb.args$cluster <- cluster[[i]]
         fwb_out <- eval(as.call(c(list(quote(fwb::fwb)), fwb.args)))
-        V <- V + sign[i] * cov(fwb_out[["t"]])
+        V <- V + sgn[i] * cov(fwb_out[["t"]])
       }
     }
   }
   else if (vcov == "BS") {
     R <- attr(vcov, "R")
 
-    glm_call$x <- FALSE
-    glm_call$y <- FALSE
-    glm_call$model <- FALSE
+    internal_model_call$x <- FALSE
+    internal_model_call$y <- FALSE
+    internal_model_call$model <- FALSE
 
     genv <- environment(fit$formula)
     if (is_not_null(weightit)) {
@@ -757,48 +774,36 @@
       data <- eval(wcall$data, wenv)
       if (is_null(data)) {
         .err(sprintf('a dataset must have been supplied to `data` in the original call to `%s()` to use `vcov = "BS"`',
-                     deparse1(wcall[[1]])))
+                     rlang::call_name(wcall)))
       }
     }
     else {
       weightit_boot <- list(weights = 1)
-      data <- eval(glm_call$data, genv)
+      data <- eval(internal_model_call$data, genv)
       if (is_null(data)) {
         .err(sprintf('a dataset must have been supplied to `data` in the original call to `%s()` to use `vcov = "BS"`',
-                     deparse1(glm_call[[1]])))
+                     rlang::call_name(model_call)))
       }
     }
 
     bootfun <- function(data, ind) {
       if (is_not_null(weightit)) {
-        wcall$data <- data[ind,]
+        wcall$data <- data[ind, ]
         wcall$s.weights <- SW[ind]
 
-        withCallingHandlers({
-          weightit_boot <- eval(wcall, wenv)
-        },
-        warning = function(w) {
-          w <- conditionMessage(w)
-          if (!startsWith(tolower(w), "some extreme weights were generated"))
-            .wrn("(from `weightit()`) ", w, tidy = FALSE)
-          invokeRestart("muffleWarning")
-        })
-        glm_call$weights <- weightit_boot$weights * SW[ind]
+        weightit_boot <- .eval_fit(wcall, envir = wenv,
+                                   warnings = c("some extreme weights were generated" = NA))
+
+        internal_model_call$weights <- weightit_boot$weights * SW[ind]
       }
       else {
-        glm_call$weights <- SW[ind]
+        internal_model_call$weights <- SW[ind]
       }
 
-      glm_call$data <- data[ind,]
+      internal_model_call$data <- data[ind, ]
 
-      withCallingHandlers({
-        fit_boot <- eval(glm_call, genv)
-      },
-      warning = function(w) {
-        w <- conditionMessage(w)
-        if (w != "non-integer #successes in a binomial glm!") .wrn("(from `glm()`) ", w, tidy = FALSE)
-        invokeRestart("muffleWarning")
-      })
+      fit_boot <- .eval_fit(internal_model_call, envir = genv,
+                            warnings = c("non-integer" = NA))
 
       fit_boot$coefficients
     }
@@ -821,7 +826,7 @@
           bootfun(data, ind)
         }))
 
-        V <- V + sign[i] * cov(boot_out)
+        V <- V + sgn[i] * cov(boot_out)
       }
     }
   }
@@ -833,170 +838,169 @@
       V <- 0
 
       for (i in seq_along(clu)) {
-        adj <- g[i]/(g[i] - 1)
+        adj <- g[i] / (g[i] - 1)
         temp <- residuals(fit, type = "dfbeta",
                           collapse = cluster[[i]],
                           weighted = TRUE)
 
-        V <- V + sign[i] * adj * crossprod(temp)
+        V <- V + sgn[i] * adj * crossprod(temp)
       }
     }
   }
   else {
-    hess <- NULL
     if (vcov == "asympt") {
       Mparts <- attr(weightit, "Mparts", exact = TRUE)
       Mparts.list <- attr(weightit, "Mparts.list", exact = TRUE)
     }
+    else {
+      Mparts <- Mparts.list <- NULL
+    }
 
     psi_out <- function(Bout, w, Y, Xout, SW, offset) {
       fit$psi(Bout, Xout, Y, w * SW, offset = offset)
     }
 
-    if (vcov == "HC0") {
-      wfun <- {
-        if (is_null(W)) function(Btreat, Xtreat, A) 1
-        else function(Btreat, Xtreat, A) W
-      }
-
-      Xtreat <- NULL
-      A <- NULL
-
-      psi <- function(B, Xout, Y, Xtreat, A, SW, offset) {
-        Bout <- B[seq_len(pout)]
-        Btreat <- NULL
-
-        w <- wfun(Btreat, Xtreat, A)
-        psi_out(B, w, Y, Xout, SW, offset)
-      }
-
-      b <- bout
-
-      psi_b <- if_null_then(fit[["gradient"]], psi(b, Xout, Y, Xtreat, A, SW, offset))
+    psi_b <- if_null_then(fit[["gradient"]],
+                          psi_out(bout, W, Y, Xout, SW, offset))
 
+    H <- {
       if (is_not_null(fit[["hessian"]])) {
-        hess <- fit$hessian
+        fit$hessian
       }
       else if (inherits(fit, "ordinal_weightit")) {
-        hess <- .get_hess_ordinal(fit)
+        .get_hess_ordinal(fit)
       }
       else if (inherits(fit, "multinom_weightit")) {
-        hess <- .get_hess_multinom(fit)
+        .get_hess_multinom(fit)
+      }
+      else if (inherits(fit, "glm")) {
+        .get_hess_glm(fit)
+      }
+      else {
+        .gradient(function(Bout) {
+          colSums(psi_out(Bout, W, Y, Xout, SW, offset))
+        }, .x = bout)
       }
     }
-    else if (is_not_null(Mparts)) {
+
+    if (is_not_null(Mparts)) {
       # Mparts from weightit()
       psi_treat <- Mparts[["psi_treat"]]
       wfun <- Mparts[["wfun"]]
       Xtreat <- Mparts[["Xtreat"]]
       A <- Mparts[["A"]]
       btreat <- Mparts[["btreat"]]
+      hess_treat <- Mparts[["hess_treat"]]
+      dw_dBtreat <- Mparts[["dw_dBtreat"]]
 
-      psi <- function(B, Xout, Y, Xtreat, A, SW, offset) {
-        Bout <- B[seq_len(pout)]
-        Btreat <- B[-seq_len(pout)]
-
-        w <- wfun(Btreat, Xtreat, A)
-
-        cbind(psi_out(Bout, w, Y, Xout, SW, offset),
-              psi_treat(Btreat, A, Xtreat, SW))
+      H_treat <- {
+        if (is_not_null(hess_treat)) {
+          hess_treat(btreat, Xtreat, A, SW)
+        }
+        else {
+          .gradient(function(Btreat) {
+            colSums(psi_treat(Btreat, Xtreat, A, SW))
+          }, .x = btreat)
+        }
       }
 
-      b <- c(bout, btreat)
+      H_out_treat <- {
+        if (is_not_null(dw_dBtreat)) {
+          crossprod(psi_out(bout, 1, Y, Xout, SW, offset),
+                    dw_dBtreat(btreat, Xtreat, A, SW))
+        }
+        else {
+          .gradient(function(Btreat) {
+            w <- wfun(Btreat, Xtreat, A)
+            colSums(psi_out(bout, w, Y, Xout, SW, offset))
+          }, .x = btreat)
+        }
+      }
 
-      psi_b <- psi(b, Xout, Y, Xtreat, A, SW, offset)
+      #Using formula from Wooldridge (2010) p. 419
+      psi_b <- psi_b + psi_treat(btreat, Xtreat, A, SW) %*%
+        .solve_hessian(-H_treat, t(H_out_treat), model = "weights")
     }
-    else {
+    else if (is_not_null(Mparts.list)) {
       # Mparts.list from weightitMSM() or weightit()
       psi_treat.list <- grab(Mparts.list, "psi_treat")
       wfun.list <- grab(Mparts.list, "wfun")
       Xtreat.list <- grab(Mparts.list, "Xtreat")
       A.list <- grab(Mparts.list, "A")
       btreat.list <- grab(Mparts.list, "btreat")
+      hess_treat.list <- grab(Mparts.list, "hess_treat")
+      dw_dBtreat.list <- grab(Mparts.list, "dw_dBtreat")
 
-      psi_treat <- function(Btreat, A, Xtreat, SW) {
-        do.call("cbind", lapply(seq_along(Btreat), function(i) {
-          psi_treat.list[[i]](Btreat[[i]], A[[i]], Xtreat[[i]], SW)
+      psi_treat <- function(Btreat.list, Xtreat.list, A.list, SW) {
+        do.call("cbind", lapply(seq_along(Btreat.list), function(i) {
+          psi_treat.list[[i]](Btreat.list[[i]], Xtreat.list[[i]], A.list[[i]], SW)
         }))
       }
 
-      wfun <- function(Btreat, Xtreat, A) {
-        Reduce("*", lapply(seq_along(Btreat), function(i) {
-          wfun.list[[i]](Btreat[[i]], Xtreat[[i]], A[[i]])
+      wfun <- function(Btreat.list, Xtreat.list, A.list) {
+        Reduce("*", lapply(seq_along(Btreat.list), function(i) {
+          wfun.list[[i]](Btreat.list[[i]], Xtreat.list[[i]], A.list[[i]])
         }), init = 1)
       }
 
-      psi <- function(B, Xout, Y, Xtreat, A, SW, offset) {
-        Bout <- B[seq_len(pout)]
-        Btreat <- btreat.list
-        k <- 0
-        for (i in seq_along(btreat.list)) {
-          Btreat[[i]] <- B[-seq_len(pout)][k + seq_along(btreat.list[[i]])]
-          k <- k + length(btreat.list[[i]])
+      H_treat <- {
+        if (all(lengths(hess_treat.list) > 0L)) {
+          .block_diag(lapply(seq_along(hess_treat.list), function(i) {
+            hess_treat.list[[i]](btreat.list[[i]], Xtreat.list[[i]], A.list[[i]], SW)
+          }))
+        }
+        else {
+          .gradient(function(Btreat) {
+            Btreat.list <- .vec2list(Btreat, lengths(btreat.list))
+            colSums(psi_treat(Btreat.list, Xtreat.list, A.list, SW))
+          }, .x = unlist(btreat.list))
         }
-
-        w <- wfun(Btreat, Xtreat, A)
-
-        cbind(psi_out(Bout, w, Y, Xout, SW, offset),
-              psi_treat(Btreat, A, Xtreat, SW))
       }
 
-      b <- c(bout, unlist(btreat.list))
+      if (all(lengths(dw_dBtreat.list) > 0L)) {
+        w.list <- c(lapply(seq_along(btreat.list), function(i) {
+          wfun.list[[i]](btreat.list[[i]], Xtreat.list[[i]], A.list[[i]])
+        }), list(rep(1, length(A.list[[1L]]))))
 
-      psi_b <- psi(b, Xout, Y, Xtreat.list, A.list, SW, offset)
+        dw_dBtreat <- do.call("cbind", lapply(seq_along(btreat.list), function(i) {
+          dw_dBtreat.list[[i]](btreat.list[[i]], Xtreat.list[[i]], A.list[[i]], SW) *
+            Reduce("*", w.list[-i])
+        }))
+
+        H_out_treat <- crossprod(psi_out(bout, 1, Y, Xout, SW, offset), dw_dBtreat)
+      }
+      else {
+        H_out_treat <- .gradient(function(Btreat) {
+          Btreat.list <- .vec2list(Btreat, lengths(btreat.list))
+          w <- wfun(Btreat.list, Xtreat.list, A.list)
+          colSums(psi_out(bout, w, Y, Xout, SW, offset))
+        }, .x = unlist(btreat.list))
+      }
+
+      #Using formula from Wooldridge (2010) p. 419
+      psi_b <- psi_b + psi_treat(btreat.list, Xtreat.list, A.list, SW) %*%
+        .solve_hessian(-H_treat, t(H_out_treat), model = "weights")
     }
 
     if (is_not_null(cluster)) {
-      B <- 0
+      B1 <- 0
 
       for (i in seq_along(clu)) {
-        adj <- g[i]/(g[i] - 1)
+        adj <- g[i] / (g[i] - 1)
 
-        B <- B + sign[i] * adj * crossprod(rowsum(psi_b, cluster[[i]], reorder = FALSE))
+        B1 <- B1 + sgn[i] * adj * crossprod(rowsum(psi_b, cluster[[i]], reorder = FALSE))
       }
     }
     else {
-      B <- crossprod(psi_b)
+      B1 <- crossprod(psi_b)
     }
 
-    # Gradient of gradfun -> hessian
-    gradfun <- function(B, Xout, Y, Xtreat, A, SW, offset) {
-      colSums(psi(B, Xout, Y, Xtreat, A, SW, offset))
-    }
+    A1 <- .solve_hessian(H)
 
-    if (is_null(hess)) {
-      hess <- .gradient(gradfun,
-                        .x = b,
-                        Xout = Xout,
-                        Y = Y,
-                        Xtreat = {
-                          if (is_not_null(Mparts.list)) Xtreat.list
-                          else Xtreat
-                        },
-                        A = {
-                          if (is_not_null(Mparts.list)) A.list
-                          else A
-                        },
-                        SW = SW,
-                        offset = offset)
-    }
+    V <- A1 %*% tcrossprod(B1, A1)
 
-    A1 <- try(solve(hess, t(.chol2(B))), silent = TRUE)
-
-    if (null_or_error(A1)) {
-      .e <- conditionMessage(attr(A1, "condition"))
-      if (startsWith(.e, "system is computationally singular") ||
-          startsWith(.e, "Lapack routine dgesv: system is exactly singular")) {
-        .err('the Hessian could not be inverted, which indicates an estimation failure, likely due to perfect separation. Estimates from this model should not be trusted. Investigate the problem by refitting with `vcov = "none"`. Simplifying the model can sometimes help')
-      }
-
-      .err(.e, tidy = FALSE)
-    }
-
-    #Subset to just the outcome model coefs
-    A1 <- A1[seq_len(pout), , drop = FALSE]
-
-    V <- tcrossprod(A1)
+    # inf <- psi_b %*% A1
+    # V <- crossprod(inf)
   }
 
   colnames(V) <- rownames(V) <- names(aliased)[!aliased]
@@ -1005,7 +1009,7 @@
 }
 
 # Processes fit for output; used in glm_weightit()
-.process_fit <- function(fit, weightit = NULL, vcov, glm_weightit_call, x, y) {
+.process_fit <- function(fit, weightit = NULL, vcov, model_call, x, y) {
   if (is_not_null(weightit) && is_not_null(fit[["model"]])) {
     fit$model[["(s.weights)"]] <- weightit[["s.weights"]]
     fit$model[["(weights)"]] <- weightit[["weights"]] * weightit[["s.weights"]]
@@ -1013,7 +1017,7 @@
 
   fit$vcov_type <- attr(fit[["vcov"]], "vcov_type")
 
-  fit$call <- glm_weightit_call
+  fit$call <- model_call
 
   if (is_not_null(weightit)) {
     fit$weightit <- weightit
@@ -1027,4 +1031,40 @@
   fit[["vcov"]] <- .modify_vcov_info(fit[["vcov"]])
 
   fit
-}
\ No newline at end of file
+}
+
+.get_hess_glm <- function(fit) {
+  X <- if_null_then(fit[["x"]], model.matrix(fit))
+
+  d1mus <- fit$family$mu.eta(fit$linear.predictors)
+  varmus <- fit$family$variance(fit$fitted.values)
+
+  crossprod(X, X * (-d1mus^2 * fit$prior.weights / varmus))
+}
+
+.solve_hessian <- function(h, ..., model = "out") {
+  model <- match_arg(model, c("out", "weights"))
+
+  withCallingHandlers({
+    solve(h, ...)
+  },
+  warning = function(w) {
+    .wrn(conditionMessage(w), tidy = FALSE)
+    invokeRestart("muffleWarning")
+  },
+  error = function(e) {
+    .e <- conditionMessage(e)
+    if (grepl("system is computationally singular", .e, fixed = TRUE) ||
+        grepl("Lapack routine dgesv: system is exactly singular", .e, fixed = TRUE)) {
+      if (model == "out") {
+        .err('the Hessian for the outcome model could not be inverted, which indicates an estimation failure, possibly due to perfect separation or a model that is too complex. Estimates from this model should not be trusted. Investigate the problem by refitting with `vcov = "none"`. Simplifying the model can sometimes help')
+      }
+
+      if (model == "weights") {
+        .err('the Hessian for the weighting model could not be inverted, which indicates an estimation failure, possibly due to perfect separation or failure to converge. Consider treating the weights as fixed by setting `vcov = "HC0"`')
+      }
+    }
+
+    .err(.e, tidy = FALSE)
+  })
+}
